﻿using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.Drawing;
using System.IO;
using System.Reflection;
using System.Windows.Forms;

namespace PuzzlePlayer_Namespace
{
    // form to display the settings in
    internal class SettingForm : Form
    {
        // Public variables for all settings
        // Currently they are set to their default values
        //public static Color primaryColor = Color.FromArgb(66, 69, 73);
        //public static Color secondaryColor = Color.FromArgb(54, 57, 62);
        //public static Color tertiaryColor = Color.FromArgb(0, 0, 0);
        //public static int fontSize = 16;
        //public static string mainFontName = "Gotham";
        //public static Font mainFont = new Font(mainFontName, fontSize);
        public static FlowLayoutPanel settingsPanel;

        public SettingForm()
        {
            this.ClientSize = new Size(1115, 755);
            this.BackColor = UserDataManager.Theme.primaryColor;
            this.Text = "PuzzlePlayer";

            SetUpUI();

            foreach (Control control in Controls)
            {
               UpdateControl(control); //cheeky dw
            }
            Invalidate();
        }

        private void SetUpUI()
        {
            Theme theme = UserDataManager.Theme;
            Color primaryColor = theme.primaryColor;
            Color secondaryColor = theme.secondaryColor;
            Color tertiaryColor = theme.tertiaryColor;

            MenuStrip menuStrip = new MenuStrip
            {
                BackColor = secondaryColor,
                ForeColor = secondaryColor,
                Name = "Main menu",
                Text = "Main Menu",
                Dock = DockStyle.Top
            };

            ToolStripMenuItem menuBack = new ToolStripMenuItem
            {
                BackColor = secondaryColor,
                ForeColor = Color.Black,
                Text = "Back",
                TextAlign = ContentAlignment.BottomRight
            };

            menuBack.Click += (object o, EventArgs e) =>
            {
                this.Close();
            };

            //
            ComboBox themeSelector = new ComboBox
            {
                Text = "Theme",
                Name = "Theme",
                Height = 25,
                Width = 250,
                BackColor = secondaryColor,
                ForeColor = Color.Black,
                Margin = new Padding(20)
            };

            List<Theme> themeList = UserDataManager.GetThemes();
            foreach(Theme t in themeList)
            {
                themeSelector.Items.Add(t.name);
            }

            themeSelector.SelectedIndexChanged += (object o, EventArgs e) =>
            {
                UserDataManager.SetCurrentTheme(themeSelector.Text);
            };

            settingsPanel = new FlowLayoutPanel
            {
                Location = new Point(0, menuStrip.Height),
                Size = new Size(ClientSize.Width, ClientSize.Height - menuStrip.Height - 100),
                Anchor = AnchorStyles.Top | AnchorStyles.Left | AnchorStyles.Right | AnchorStyles.Bottom,
                FlowDirection = FlowDirection.TopDown,
                WrapContents = true,
                AutoScroll = true,
                Padding = new Padding(20),
            };

            //
            RoundedButton saveButton = new RoundedButton
            {
                Text = "Save",
                Name = "Save",
                Height = 100,
                Width = 350,
                BackColor = secondaryColor,
                ForeColor = Color.Black,
                Location = new Point(10, ClientSize.Height - 110),
                Anchor = AnchorStyles.Left | AnchorStyles.Bottom,
                FlatStyle = FlatStyle.Flat,
                Margin = new Padding(10)
            };

            saveButton.Click += (object o, EventArgs e) =>
            {
                this.BackColor = UserDataManager.Theme.primaryColor;

                foreach (Control control in Controls)
                {
                    UpdateControl(control);
                }

                settingsPanel.BackColor = UserDataManager.Theme.primaryColor;
                Invalidate();
            };

            //
            this.Controls.Add(menuStrip);
            this.Controls.Add(settingsPanel);
            this.Controls.Add(saveButton);

            settingsPanel.Controls.Add(themeSelector);

            menuStrip.Items.Add(menuBack);
            
            menuStrip.BringToFront();
            themeSelector.BringToFront();
            saveButton.BringToFront();
        }

        public static void UpdateControl(Control control)
        {
            Theme theme = UserDataManager.Theme;
            Color primaryColor = theme.primaryColor;
            Color secondaryColor = theme.secondaryColor;
            Color tertiaryColor = theme.tertiaryColor;

            if (control is not FlowLayoutPanel)
            {
                control.BackColor = secondaryColor;
                control.ForeColor = tertiaryColor;
            }
            else
            {
                control.BackColor = primaryColor;
            }

            control.Font = UserDataManager.MainFont;

            foreach (Control childControl in control.Controls)
            {
                UpdateControl(childControl);
            }

            if (control is MenuStrip menuStrip)
            {
                foreach (ToolStripItem item in menuStrip.Items)
                {
                    UpdateToolStripSettings(item);
                }
            }
        }

        public static void UpdateToolStripSettings(ToolStripItem item)
        {
            Theme theme = UserDataManager.Theme;
            Color primaryColor = theme.primaryColor;
            Color secondaryColor = theme.secondaryColor;
            Color tertiaryColor = theme.tertiaryColor;

            item.BackColor = secondaryColor;
            item.ForeColor = tertiaryColor;
            item.Font = UserDataManager.MainFont;

            if (item is ToolStripMenuItem menuItem)
            {
                foreach (ToolStripItem subItem in menuItem.DropDownItems)
                {
                    UpdateToolStripSettings(subItem);
                }
            }
        }

        public static Image GetEmbeddedImage(string imageWithExtention)
        {
            string imagePath = "PuzzlePlayer_Namespace.Resources." + imageWithExtention;
            Stream stream = Assembly.GetExecutingAssembly().GetManifestResourceStream(imagePath);
            Image img = Image.FromStream(stream);
            stream.Dispose();
            return img;
        }
    }
}
